import pygame
import utils

class Floor:
    def __init__(self, y, startx, stopx):

        self.trampoline = False
        self.y = y
        self.startx = startx
        self.stopx = stopx
        self.clr = 0

    def intersect(self, x):
        if ( x >= self.startx and x < self.stopx ) or ( (x+1) > self.startx and (x+1) <= self.stopx ) :
            return True
        else:
            return False

    def draw(self, surface):
        pygame.draw.line(surface, self.clr, \
                         utils.to_real_coords((self.startx, self.y)), \
                         utils.to_real_coords((self.stopx, self.y)), 3)

    def does_sideway_collision_with(self, old_x, old_y, x, y):
        if ((old_x > self.stopx) and (x < self.stopx)) or ((old_x < (self.startx-1)) and (x > self.startx -1)): # new intersection in x
            if ((self.y - y) < 1) and ((self.y - y) > 0):
                return True
        return False
        

class Trampoline(Floor):
    def __init__(self, y, startx, stopx):
        Floor.__init__(self, y, startx, stopx)
        self.trampoline = True
        self.clr = 0xff0000

        




class Wall:
    def __init__(self, x, y, height):
        self.height = height
        self.x = x
        self.y = y

    def draw(self, surface):
        pygame.draw.line(surface, 0, \
                         utils.to_real_coords((self.x, self.y)), \
                         utils.to_real_coords((self.x, self.y+self.height)), 3)


    def intersect(self, old_x, old_y, new_x, new_y):
        if (new_y >= self.y and new_y < ( self.y + self.height)) or \
               ((new_y+0.7) > self.y and (new_y+1) <= ( self.y + self.height)) :
            #print old_x,":",self.x,":",new_x
            if ( (old_x+1) <= self.x and (new_x+1) >= self.x ):
                return True
            elif ( old_x >= self.x and new_x <= self.x ):
                return True
                
        return False


    def is_over_object(self, o_x, o_y):
        if ((self.x-o_x) < 1) and ((self.x-o_x) > 0) :
            if ( (self.y) <= o_y+1) and ((self.y) > o_y-0.5):
                return True



    def is_under_object(self, o_x, o_y):
        if ((self.x-o_x) < 1) and ((self.x-o_x) > 0) :
            if ( (self.y+self.height) > (o_y+0.1)) and ((self.y+self.height) < o_y+0.5):
                return True
        
