import pygame
import data
import utils

class GameObject(pygame.sprite.Sprite):

    MAXSPEED = 0.2
    
    
    def __init__(self, x, y, parent, sprts, surface):

        self.surface = surface
        self.thud_snd = data.load_sound("thud.wav")
        self.uh_snd = data.load_sound("thud.wav")
        #set up sprite
        pygame.sprite.Sprite.__init__(self)
        self.images = []
        for sprt in sprts:
            img, rect = data.load_image(sprt)
            self.images.append(img)
        self.rect = rect
        #self.image, self.rect = data.load_image(sprt)

        self.startx = x
        self.starty = y
        self.x = x
        self.y = y
        self.accel_x = 0
        self.accel_y = -0.01
        self.accel_run = 0
        self.v_x = 0
        self.v_y = 0
        self.parent = parent
        self.wallhug_left = 0
        self.wallhug_right = 0

        self.switchcount = 6 # switch 10 times / sec
        self.curimg = 0
        self.image = self.images[self.curimg]

    def is_in_air(self):
        floor = self.parent.get_intersection_below(self.x, self.y)
        if floor:
            if floor.y < self.y:
                return True
        else:
            return True
        return False

    def update(self):

        #sprite handling
        #OPTIMIZE THIS!!!!!!
        if self.is_in_air():
            if self.wallhug_right or self.wallhug_left:
                self.curimg = 6
            #if self.curimg != 1:
            #    self.curimg = 1
            #    self.image = self.images[self.curimg]
        else:
            if self.v_x:
                self.switchcount -= 1
                if self.switchcount == 0:
                    self.switchcount = 6
                    self.curimg += 1
                    if self.curimg >= 8:
                        self.curimg = 0
            else:
                if self.curimg != 8:
                    self.curimg = 8


        if (self.v_x < 0) or self.wallhug_left:
            self.image = pygame.transform.flip(self.images[self.curimg], 1, 0)
        else:
            self.image = self.images[self.curimg]
        #END OF OPTIMIZE NEEDED

        
        old_x, old_y = self.x, self.y

        
        self.v_x += self.accel_x #+ self.accel_run

        #check for intersections with roof
        floor_above = self.parent.get_intersection_above(self.x, self.y)

        #check for intersections with floor
        floor = self.parent.get_intersection_below(self.x, self.y)
        if floor:
            if floor.y < self.y:
                #above a floor

                #update fallspeed
                #if wallhugging, then no acceleration!
                if not ( (self.wallhug_left) or (self.wallhug_right) ):
                    self.v_y += self.accel_y 
                else:
                    if self.v_y > 0:
                        self.v_y += self.accel_y*2
                    else:
                        self.v_y += self.accel_y/3 


                if self.v_x < GameObject.MAXSPEED and \
                    self.v_x > -GameObject.MAXSPEED:
                    self.v_x += self.accel_run/4
                    if self.wallhug_left and self.v_x < 0:
                        self.v_x = 0
                    if self.wallhug_right and self.v_x > 0:
                        self.v_x = 0
                
                #update y position
                self.y += self.v_y

                #dont fall through floor!
                if self.y < floor.y:
                    self.y = floor.y
                    self.v_y = 0

                if not self.is_in_air(): self.thud_snd.play()
                    
            else:
                #at a floor

                #at a trampoline floor
                if floor.trampoline:
                    self.v_y += 0.3
                else:
                    pass
                
                if self.v_x < GameObject.MAXSPEED and \
                       self.v_x > -GameObject.MAXSPEED:
                    self.v_x += self.accel_run
                

                #Friction
                if not self.accel_run or self.accel_run*self.v_x < 0:
                    tresh = 0.03  #treshhold
                    if self.v_x > tresh:
                        self.v_x -= tresh
                    elif self.v_x < -tresh:
                        self.v_x += tresh
                    else:
                        self.v_x = 0
                
                #jump
                if self.v_y < 0:
                    self.v_y = 0
                elif self.v_y > 0:
                    self.y += self.v_y
                    

                
        else:
            #no floor!

            #update fallspeed
            #if wallhugging, then no acceleration!
            if not ( (self.wallhug_left) or (self.wallhug_right) ):
                self.v_y += self.accel_y 
            else:
                if self.v_y > 0:
                    self.v_y += self.accel_y*2
                else:
                    self.v_y += self.accel_y/3 


            #update position
            self.y += self.v_y

            if not self.is_in_air(): self.thud_snd.play()


        #make sure you dont fall through the roof! :)
        if floor_above:
            if ((old_y + 0.9) < floor_above.y) and ((self.y+0.9) >= floor_above.y):
                self.uh_snd.play()
                self.y = floor_above.y - 0.9
                self.v_y = 0


        #move!
        if (self.wallhug_left and self.v_x < 0 ) or (self.wallhug_right and self.v_x > 0):
            pass
        else:
            self.x += self.v_x
        

        #check wall intersection
        wall = self.parent.get_intersecting_wall(\
            old_x, old_y, self.x, self.y)

        if wall:
            #print "INTERSECTING WALL!"
            #print "w_x",wall.x
            #print "old_x",old_x
            #print "new_x",self.x

            #self.v_y = 0
            if (old_x >= wall.x) and (self.x <= wall.x) :
                if not self.wallhug_right:
                    self.thud_snd.play()
                self.wallhug_right = True
                self.x = wall.x+0
                
            elif ((old_x+1) <= wall.x) and ((self.x+1) >= wall.x):
                if not self.wallhug_left:
                    self.thud_snd.play()
                self.wallhug_left = True
                self.x = wall.x-1


            else:
                "no hug"
            self.accel_x = 0
            self.v_x = 0

        #check wall intersection along Y-axis
        wall = self.parent.get_wall_under(self.x, self.y)
        if wall:
            #wall under player. stop movement!
            if self.is_in_air():
                self.y = old_y
                self.v_y = 0


        wall = self.parent.get_wall_over(self.x, self.y)

        if wall:
            #wall over player. stop movement!
            if self.v_y > 0:
                self.v_y = 0


        #check for sideways collision with floors
        if self.parent.does_sideway_collision(old_x, old_y, self.x, self.y):
            self.v_x = 0


        #check if wallhugging and suddenly no wall!
        if self.wallhug_right:
            wall = self.parent.get_intersecting_wall(\
                self.x, self.y, self.x-0.5, self.y)
            if not wall:
                self.wallhug_right = 0

        if self.wallhug_left:
            wall = self.parent.get_intersecting_wall(\
                self.x, self.y, self.x+1.5, self.y)
            if not wall:
                self.wallhug_left = 0

        #sprite stuff
        ##move to new pos
        self.rect.bottomleft = utils.to_real_coords((self.x, self.y))#to_real_coords((20,10))


        #check if at goal
        dx = (self.x+0.5)-self.parent.end[0]
        dy = (self.y+0.5)-self.parent.end[1]


        #ugly off screen detection...
        if self.y < -5 or self.x < -5:
            self.x = self.startx
            self.y = self.starty
            
        if (dx**2+dy**2) < 1:
            self.parent.win_level()


class Hero(GameObject):
    def __init__(self, x, y, parent, sfc):
        GameObject.__init__(self, x, y, parent, ["walk%d.png"% n for n in range(1,9)]+["stand.png"], sfc)



    def moveStop(self):
        self.accel_run = 0

    def moveDown(self):
        print self.accel_x
        print self.v_x
        print self.accel_run
    def moveUp(self):
        if self.wallhug_left:
           self.wallhug_left = False
        if self.wallhug_right:
           self.wallhug_right = False
        if not self.is_in_air():
            self.v_y += 0.18
    def moveLeft(self, stop=0):
        if self.wallhug_left:
            self.wallhug_left = False
        if stop:
            if self.accel_run < 0:
                self.accel_run = 0
        else:
            self.accel_run = -0.008
    def moveRight(self, stop = 0):
        if self.wallhug_right:
            self.wallhug_right = False
        if stop:
            if self.accel_run > 0:
                self.accel_run = 0
        else:
            self.accel_run = 0.008
        #self.accel_run += 0.008-stop*0.016

    def dash(self):
        if self.is_in_air() and ( self.wallhug_right or self.wallhug_left ):
            self.wall_dash()
        elif not self.is_in_air():
            self.run_dash()
        

    def wall_dash(self):
        if self.wallhug_right:
            self.v_x += 0.2
            self.v_y += 0.2
            self.wallhug_right = False
            
        elif self.wallhug_left:
            self.v_x -= 0.2
            self.v_y += 0.2
            self.wallhug_left = False


    def run_dash(self):
        if self.v_x > 0:
            self.v_x += 0.1
            self.v_y += 0.1
        elif self.v_x < 0:
            self.v_x -= 0.1
            self.v_y += 0.1
