import pygame
import fontmanager
import data
import env

#translate from "ingame" coords to "real" coords
def to_real_coords((x,y)):
    #ingame has 40 16px tiles in width. 0-39
    #ingame has 20 24px tiles in height. 0-10
    #real has 640 x 480 px
    #real counts from top. ingame counts from bottom
    return (x*16, 480-y*24)


class Grid:
    def __init__(self):
        pass

    def draw(self, surface):
        for x in range(40):
            for y in range(20):
                pygame.draw.circle(surface, 0, 
                                   to_real_coords((x, y)), \
                                   1, 1) 

def show_message(msg, surface):
    fontmgr = fontmanager.mFontManager((('44v2.ttf', 20),('44v2.ttf', 16)))
    background = data.load_image("bg.jpg")[0].convert()
    surface.blit(background, (0, 0))

    fontmgr.Draw(surface, '44v2.ttf', 20, msg,
                      pygame.Rect(0,0,640,480), (150,42,42), 'center', 'center', True)

    running = 1
    while running:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit(1)
            elif event.type == pygame.KEYDOWN:
                running = 0
        
        pygame.display.flip()

def show_finish_screen(surface, scores):
    fontmgr = fontmanager.mFontManager((('44v2.ttf', 20),('44v2.ttf', 24)))
    background = data.load_image("won.jpg")[0].convert()
    surface.blit(background, (0, 0))

    fontmgr.Draw(surface, '44v2.ttf', 24, "Your times:",
                 pygame.Rect(0,130,640,30), (150,42,42), 'center', 'center', True)

    for l in range(1,9):
        fontmgr.Draw(surface, '44v2.ttf', 20, "Level %d: %.2f sec" %(l, scores[l-1]),
                     pygame.Rect(40,180+l*16,320,16), (150,42,42), 'left', 'center', True)
    for l in range(9,17):
        fontmgr.Draw(surface, '44v2.ttf', 20, "Level %d: %.2f sec" %(l, scores[l-1]),
                     pygame.Rect(360,180+(l-8)*16,320,16), (150,42,42), 'left', 'center', True)

    fontmgr.Draw(surface, '44v2.ttf', 24, "Total: %.2f sec"%sum(scores),
                 pygame.Rect(0,350,640,40), (150,42,42), 'center', 'center', True)
    
    running = 1
    while running:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                sys.exit(1)
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_RETURN or event.key == pygame.K_SPACE or event.key == pygame.K_ESCAPE:
                    running = 0
        
        pygame.display.flip()

def merge_walls(w1, w2):
    print "did a merge"
    x = w1.x
    y1 = min(w1.y, w2.y)
    y2 = max(w1.y+w1.height, w2.y+w2.height)
    return env.Wall(x, y1, y2-y1)

def fix_level(building):
    #remove walls with no height and floors with no width
    walls = [w for w in building.walls if w.height]
    floors = [f for f in building.floors if f.startx != f.stopx]

    #walls
    new_walls = []
    for wall1 in walls:
        for wall2 in walls:
            if not (wall1 is wall2):
                if wall1.x == wall2.x: #at same x

                    #merge walls
                    if (wall1.y >= wall2.y) and (wall1.y <= (wall2.y+wall2.height)) or \
                       (wall2.y >= wall1.y) and (wall2.y <= (wall1.y+wall1.height)):
                        miny = min(wall1.y, wall2.y)
                        maxy = max(wall1.y+wall1.height, wall2.y+wall2.height)
                        height = maxy-miny
                        wall1.y = miny
                        wall1.height = height
                        walls.pop(walls.index(wall2))
    #floors & trampolines
    for floor1 in floors:
        for floor2 in floors:
            if not (floor1.trampoline == floor2.trampoline): #both must be same type
                continue
            if not (floor1 is floor2):
                if floor1.y == floor2.y: #at same y

                    #merge floors
                    if (floor1.startx >= floor2.startx) and (floor1.startx <= floor2.stopx) or \
                       (floor2.startx >= floor1.startx) and (floor2.startx <= floor1.stopx):
                        minx = min(floor1.startx, floor2.startx)
                        maxx = max(floor1.stopx, floor2.stopx)
                        print minx,maxx
                        print floor1.startx, floor1.stopx
                        floor1.startx = minx
                        floor1.stopx = maxx
                        print floor1.startx, floor1.stopx
                        floors.pop(floors.index(floor2))

    building.walls = walls
    building.floors = floors
                        
                    
    
